'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

require('core-js/fn/array/is-array');

var _tv = require('tv4');

var _tv2 = _interopRequireDefault(_tv);

var _sert = require('sert');

var _sert2 = _interopRequireDefault(_sert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var SertSchema = _extends({}, _sert2.default);

function _flatten(error) {
    if (Array.isArray(error.subErrors) && error.subErrors.length > 0) {
        error.message = error.message + ' (Possible reasons: ' + error.subErrors.map(function (err) {
            return _flatten(err).message;
        }).join(' / ') + ')';
    }

    return error;
}

function _pathMessage(error) {
    return error.message + ' at path "' + (error.dataPath || '/') + '"';
}

/**
 * Validate an object by a schema
 * @param {object} schema Validation schema
 * @param {object} subject Object to validate
 * @param {object} options Error message options
 * @param {string} [options.message] Error message
 * @param {string} [options.status] Error message status
 * @return {undefined|Error} Error if invalid, undefined if valid
 */
SertSchema.validate = function (schema, subject) {
    var _ref2;

    var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        _ref$message = _ref.message,
        message = _ref$message === undefined ? null : _ref$message,
        _ref$status = _ref.status,
        status = _ref$status === undefined ? null : _ref$status;

    if ((typeof schema === 'undefined' ? 'undefined' : _typeof(schema)) !== 'object') {
        throw new Error('Invalid schema');
    }

    var result = _tv2.default.validateMultiple(subject, schema);
    if (result.valid) {
        return undefined;
    }

    var errors = (_ref2 = []).concat.apply(_ref2, _toConsumableArray(result.errors.map(_flatten)));

    // Generate error message
    var err = new Error(message || 'Invalid ' + (schema.title || 'object'));

    err.summary = errors.length > 1 ? _pathMessage(errors[0]) + ', and ' + (errors.length - 1) + ' other validation ' + (errors.length > 2 ? 'errors' : 'error') + '.' // eslint-disable-line max-len
    : _pathMessage(errors[0]);

    err.details = (errors.length > 1 ? 'All validation errors: ' : '') + errors.map(_pathMessage).join('. ') + '.';

    if (status) {
        err.status = status;
    }

    err.messages = errors.map(_pathMessage);

    err.propMessages = errors.reduce(function (propMessages, error) {
        var existingPathErrors = propMessages[error.dataPath] || [];
        return _extends({}, propMessages, _defineProperty({}, error.dataPath, [].concat(_toConsumableArray(existingPathErrors), [error.message])));
    }, {});

    return err;
};

/**
 * Assert an object to be valid by a schema, throwing an error if not valid
 * @param {object} schema Validation schema
 * @param {object} subject Object to validate
 * @param {object} options Error message options
 * @param {string} [options.message] Error message
 * @param {string} [options.status] Error message status
 * @return {object} Subject (for call chains)
 */
SertSchema.assertValid = function (schema, subject) {
    var _ref3 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        _ref3$message = _ref3.message,
        message = _ref3$message === undefined ? null : _ref3$message,
        _ref3$status = _ref3.status,
        status = _ref3$status === undefined ? null : _ref3$status;

    var res = SertSchema.validate(schema, subject, { message: message, status: status });
    if (res instanceof Error) {
        throw res;
    }
    return subject;
};

/**
 * Validate an object by a schema
 * @param {object} schema Validation schema
 * @param {object} subject Object to validate
 * @return {bool} True if valid, false if invalid
 */
SertSchema.isValid = function (schema, subject) {
    return _tv2.default.validate(subject, schema);
};

exports.default = SertSchema;