'use strict';

var _ava = require('ava');

var _ava2 = _interopRequireDefault(_ava);

var _SertSchema = require('../SertSchema');

var _SertSchema2 = _interopRequireDefault(_SertSchema);

var _book = require('testdata/got/schemas/book.json');

var _book2 = _interopRequireDefault(_book);

var _book3 = require('testdata/got/books/book1.json');

var _book4 = _interopRequireDefault(_book3);

var _bookInvalid = require('testdata/got/books/bookInvalid.json');

var _bookInvalid2 = _interopRequireDefault(_bookInvalid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _ava2.default)('.validate return undefined for valid subjects', function (t) {
    t.is(_SertSchema2.default.validate(_book2.default, _book4.default), undefined);
});

(0, _ava2.default)('.validate return error for invalid subjects', function (t) {
    var err1 = _SertSchema2.default.validate(_book2.default, _bookInvalid2.default);
    t.true(err1 instanceof Error);

    t.true(['summary', 'details', 'messages', 'propMessages'].every(function (prop) {
        return Boolean(err1[prop]);
    }));

    t.is(err1.message, 'Invalid Book');
    t.is(err1.status, undefined);
    t.is(err1.summary, 'Additional properties not allowed at path "/publishedAt"');
    t.is(err1.details, 'Additional properties not allowed at path "/publishedAt".');
    t.is(err1.status, undefined);

    // With message and status
    var err2 = _SertSchema2.default.validate(_book2.default, _bookInvalid2.default, { message: 'Wrong book.', status: 418 });
    t.true(err2 instanceof Error);
    t.true(['status', 'summary', 'details', 'messages', 'propMessages'].every(function (prop) {
        return Boolean(err2[prop]);
    }));
    t.is(err2.message, 'Wrong book.');
    t.is(err2.summary, 'Additional properties not allowed at path "/publishedAt"');
    t.is(err2.details, 'Additional properties not allowed at path "/publishedAt".');
    t.is(err2.status, 418);
});

(0, _ava2.default)('.assertValid return the subject', function (t) {
    var res = _SertSchema2.default.assertValid(_book2.default, _book4.default);
    t.is(res, _book4.default);
});

(0, _ava2.default)('.assertValid pass valid subjects', function (t) {
    _SertSchema2.default.assertValid(_book2.default, _book4.default);
});

(0, _ava2.default)('.assertValid throw error for invalid subjects', function (t) {
    t.throws(function () {
        return _SertSchema2.default.assertValid(_book2.default, _bookInvalid2.default);
    });
});

(0, _ava2.default)('.isValid return true for valid subjects', function (t) {
    t.true(_SertSchema2.default.isValid(_book2.default, _book4.default));
});

(0, _ava2.default)('.isValid return false for invalid subjects', function (t) {
    t.false(_SertSchema2.default.isValid(_book2.default, _bookInvalid2.default));
});